﻿/*********************************************
 *           RO表格控件适配器                *
 *                 Update Date：2013-8-7     *
**********************************************/

/**
* @class Grid操作类
* @constructor
* @param gridHtml5Flag {Boolean} 是否开启GridHtml5
* @param designMode {Boolean} true - 设计模式（无数据区域概念）；false - 常规模式
* @param scrollBar {verti,horiz} 表格滚动条
* @param showProportion {Int} 表格缩放比，默认为1（100%）
* @returns 返回一个Grid操作类实例
*/
RO.GridAdapter = function (gridHtml5Flag, designMode, scrollBar, showProportion) {
    this.gridHtml5Flag = gridHtml5Flag;
    this.designMode = !!designMode;
    this.grid = null;
    this.widgetManager = {};
    this.scrollBar = scrollBar;
    Draw2D.setShowProportion(showProportion === undefined || showProportion <= 0 ? 1 : parseFloat(showProportion));
};

/**
* 获取表格浮动类型
* @returns {String} 表格浮动类型【RO.Enum.FormRegionKind枚举】
*/
RO.GridAdapter.prototype.getFormRegionKind = function () {
    return this.getOptionByName("FormRegionKind");
};

/**
* 加载表格
* 通过gridHtml5Flag适配this.grid对象
* @param shell {Object} Grid容器
* @param formData {JsonObject} 表格数据
*/
RO.GridAdapter.prototype.loadGrid = function (shell, formData, widgetConfig) {
    if (!RO.BasicFun.isObject(shell)) {
        RO.BasicFun.alertParamCheck("shell", "loadGrid");
    }
    if (!RO.BasicFun.isObject(formData)) {
        RO.BasicFun.alertParamCheck("formData", "loadGrid");
    }

    if (this.gridHtml5Flag) {
     //   if (RO.GridAdapter.widgetReg) {
            RO.GridAdapter.widgetReg(this, widgetConfig);
       // }
        this.grid = new RO.GridHtml5(shell, formData, this.widgetManager, this.designMode, this.scrollBar);
        this.grid.setCopyFunction(this.grid.copy, this.grid);
        this.grid.setPasteFunction(this.grid.paste, this.grid);
        shell.trigger(RO.Enum.Event.onFormDataLoaded);
    }
  //  window.currentGrid = this;
};

/**
* 异步加载表格数据，支持GridHtml5
* @param asynFormData {JsonObject} 表格数据
* @param begin {Int} 异步加载的起始行索引
* @param callback {Function} 回调函数，数据加载完调用，比如“移除进度条”
*/
RO.GridAdapter.prototype.asynLoadData = function (asynFormData, begin, callback, callbackArgs) {
    if (RO.BasicFun.isNullOrUndefined(asynFormData)) {
        RO.BasicFun.alertParamCheck("asynFormData", "asynLoadData");
    }
    this.grid.asynLoadData(asynFormData, begin, callback, callbackArgs);
};

/**
* 通过坐标获取单元格对象
* @param colIndex {Int} 单元格所在列号
* @param rowIndex {Int} 单元格所在行号
* @return {Object}
* @exception 目标单元格不存在
*/
RO.GridAdapter.prototype.getCell = function (colIndex, rowIndex) {
    //if (!RO.BasicFun.isInt(colIndex)) {
    //    RO.BasicFun.alertParamCheck("colIndex", "getCell");
    //}
    //if (!RO.BasicFun.isInt(rowIndex)) {
    //    RO.BasicFun.alertParamCheck("rowIndex", "getCell");
    //}
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    var cell = this.grid.getCell(colIndex, rowIndex);
    if (cell == undefined) {
        return null;
    }
    return new RO.GridAdapter.Cell(cell);
};

/**
* 获取表格中第一个可编辑的单元格
* @param colFirst {Boolean} 可选参数，是否列优先，不传递时默认为行优先
* @return {Object}
*/
RO.GridAdapter.prototype.getFirstEditableCell = function (isColFirst) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    var cell = this.grid.getFirstEditableCell(isColFirst);
    if (cell == null)
        return null;
    return new RO.GridAdapter.Cell(cell);
};

/**
* 获取当前单元格
* @return {Object}
* @exception 不存在当前单元格
*/
RO.GridAdapter.prototype.getCurrentCell = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    var cell = this.grid.getCurrentCell();
    if (cell == undefined) {
        return null;
    }
    return new RO.GridAdapter.Cell(cell);
};

/**
* 获取选区内所有单元格
* @return {Array}
*/
RO.GridAdapter.prototype.getSelectedCells = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    var cells = this.grid.getSelectedCells();
    var cellArray = [];
    for (var i = 0; i < cells.length; i++) {
        if (cells[i]) {
            var cell = new RO.GridAdapter.Cell(cells[i]);
            if (jQuery.inArray(cell, cellArray) == -1) {
                cellArray.push(cell);
            }
        }
    }
    return cellArray;
};

RO.GridAdapter.prototype.getSelections = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return [];
    }
    return this.grid.getSelections();
};

RO.GridAdapter.prototype.setSelections = function (selections) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return;
    }
    this.grid.setSelections(selections);
};

/**
* 保存表格数据
* @param submitAllFlag {bool} 是否全部保存
* @param updateAfterSave {bool} 保存后是否更新表格数据
* @param storeTreeData {bool} 是否存储树形结构
*/
RO.GridAdapter.prototype.saveFormData = function (submitAllFlag, updateAfterSave,storeTreeData) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    return this.grid.storeData(submitAllFlag, updateAfterSave, storeTreeData);
};
/**
* 保存表格表样
* @noCompressed {Boolen} 是否压缩数据
*/
RO.GridAdapter.prototype.saveFormStyle = function (noCompressed) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    return this.grid.storeStyle(noCompressed);
};

/**
* 设置表格是否可编辑
* @param flag {Boolean} 是否可编辑
*/
RO.GridAdapter.prototype.setCanEdit = function (flag) {
    if (!RO.BasicFun.isBoolean(flag)) {
        RO.BasicFun.alertParamCheck("flag", "setCanEdit");
    }
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return;
    }
    this.grid.setCanEdit(flag);
};

/**
* 返回表格是否可编辑
* @return {Boolean}
*/
RO.GridAdapter.prototype.getCanEdit = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return false;
    }
    return this.grid.getCanEdit();
};

/**
* 设置回车切换当前单元格方向
* @param enterDirection {String} RO.Enum.EnterDirection
*/
RO.GridAdapter.prototype.setEnterDirection = function (enterDirection) {
    if (!RO.BasicFun.isString(enterDirection)) {
        RO.BasicFun.alertParamCheck("enterDirection", "setEnterDirection");
    }
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return;
    }
    this.grid.setEnterDirection(enterDirection);
};

/**
* 返回回车切换当前单元格方向
* @return {String}
*/
RO.GridAdapter.prototype.getEnterDirection = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return;
    }
    return this.grid.getEnterDirection();
};

/**
* 按方向跳转至下一单元格
* @param direction {String} RO.Enum.EnterDirection
* @param passReadonly {Bool} 是否跳过只读单元格
*/
RO.GridAdapter.prototype.doKeyDirection = function (direction, passReadonly) {
    this.grid && this.grid.doKeyDirection(direction, passReadonly);
};

/*表格行上移 下移*/
RO.GridAdapter.prototype.moveUp = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return;
    }
    this.grid.moveUp();

};
RO.GridAdapter.prototype.moveDown = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return;
    }
    this.grid.moveDown();
};

/**
* 表格行升级
* @param rowList {Array} 操作行坐标列表
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.prototype.upGradeTreeNode = function (rowList) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return;
    }
    this.grid.upGradeTreeNode(rowList);

};

/**
* 表格行降级
* @param rowList {Array} 操作行坐标列表
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.prototype.downGradeTreeNode = function (rowList) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return;
    }
    this.grid.downGradeTreeNode(rowList);
};

RO.GridAdapter.prototype.getGrade = function (rowIndex) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    return this.grid.getGrade(rowIndex);
};

/**
* 插入行/列操作
* @param count {Int} 插入行/列数
* @param index {Int} 待插入的行/列号；无参数时，插入当前单元格所在行/列
* @param mode {Boolean} RO.Enum.OperationMode  设计模式使用
* @param copyIndexes {Array} 插入行模板索引【null值时，使用模板行或者默认行复制】,暂去除
* @param depth {Int} 插入行级次
* @exception index值不存在
* @returns {Bool} 是否成功
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.prototype.insert = function (count, index, operationMode, depth, isAuto) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        throw new Error("this.grid尚未加载！");
    }
    return !!this.grid.insert(count, index, operationMode, null, null, depth, isAuto);
};
/**
* 删除行/列操作
* @param count {Int} 删除行/列数
* @param index {Int}，删除的行/列号；无参数时，删除当前选中单元格所在行/列
* @param mode {Boolean} RO.Enum.OperationMode  设计模式使用
* @exception index值不存在
* @returns {Bool} 是否成功
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.prototype.delete = function (count, index, operationMode) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        throw new Error("this.grid尚未加载！");
    }
    return !!this.grid.delete(count, index, operationMode);
};

/**
* 设置焦点单元格
* @param cell {Object} 目标单元格对象
* @param noEvent {Bool} 是否对外触发事件
*/
RO.GridAdapter.prototype.setFocus = function (cell, noEvent) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return;
    }
    if (cell) {
        this.grid.setCurrentCell(cell.getColIndex(), cell.getRowIndex(), noEvent);
      //  window.currentGrid = this;
    } else {
        this.grid.blur();
    }
};

/**
* 获取指定数据区域定义
* @param regId {Int} 区域ID
* @return {RegionKind,RegionTop,RegionLeft,RegionBottom,RegionRight,RegionId,FloatDataCount,DefaultCount,RegionExtension}
*/
RO.GridAdapter.prototype.getDataRegion = function (regId) {
    return this.grid.getDataRegion(regId);
};

/**
* 获取指定数据区域中记录数据记录父子关系的对象
* @param regId {Int} 区域ID
* @return 数据区域中记录数据记录父子关系的对象，其中属性为子记录的数据行号，值为父记录的数据行号
*/
RO.GridAdapter.prototype.getRecordRelations = function (regId) {
    return this.grid.getRecordRelations(regId);
};

/**
* 获取所有数据区域。
* @returns {DataRegion} 数据区域。
*/
RO.GridAdapter.prototype.getDataRegions = function () {
    return this.grid.getDataRegions();
};

/**
* 根据option名称设置option值。
* @param {object} key option名称。
* @param {object} value option值。
*/
RO.GridAdapter.prototype.setOptionByName = function (key, value) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return;
    }
    if (RO.BasicFun.isNullOrUndefined(key)) {
        RO.BasicFun.alertParamCheck("key", "setOptionByName");
    } else {
        this.grid.setOptionByName(key, value);
    }
};


/**
* 根据option名称设置option值。
* @param {object} key option名称。
* @param {object} value option值。
*/
RO.GridAdapter.prototype.getOptionByName = function (key) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    if (RO.BasicFun.isNullOrUndefined(key)) {
        RO.BasicFun.alertParamCheck("key", "getOptionByName");
        return null;
    } else {
        return this.grid.getOptionByName(key);
    }
};

/**
* 锁定表头操作
* @param {Int} col 锁表头列号，可选
* @param {Int} row 锁表头行号，可选
* @returns {Boolean} 是否成功锁定表头
*/
RO.GridAdapter.prototype.headerLock = function (col, row) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return false;
    }
    return this.grid.headerLock(col, row);
};

/**
* 获取锁定表头行列信息
* @returns {Object} {col: 列索引, row: 行索引}
*/
RO.GridAdapter.prototype.getHeaderLockInfo = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    var gridData = this.grid.gridModel.gridData;
    return { col: gridData.getColHeaderCount(), row: gridData.getRowHeaderCount() };
};

/**
* 解锁表头操作
* @returns {Boolean} 是否成功解锁表头
*/
RO.GridAdapter.prototype.headerUnlock = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return false;
    }
    return this.grid.headerUnlock();
};
/**
* 是否已锁定表头
* @returns {Boolean} 是否已锁定表头
*/
RO.GridAdapter.prototype.isHeaderLocked = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return false;
    }
    return this.grid.isHeaderLocked();
};

/**
* 指定行是否隐藏
* @param {Int} rowIndex 行号
* @returns {Boolean} 指定行是否为隐藏
*/
RO.GridAdapter.prototype.isRowHidden = function (rowIndex) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    if (!RO.BasicFun.isNumber(rowIndex)) {
        RO.BasicFun.alertParamCheck("rowIndex", "isRowHidden");
    }
    return this.grid.isRowHidden(rowIndex);
};
/**
* 指定列是否隐藏
* @param {Int} colIndex 列号
* @returns {Boolean} 指定列是否为隐藏
*/
RO.GridAdapter.prototype.isColHidden = function (colIndex) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    if (!RO.BasicFun.isNumber(colIndex)) {
        RO.BasicFun.alertParamCheck("colIndex", "isColHidden");
    }
    return this.grid.isColHidden(colIndex);
};

/**
* 设置行显示/隐藏
* @param row {Int} 行号
* @visible {Bool} 是否显示
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.prototype.setRowVisible = function (row, visible) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    if (!RO.BasicFun.isNumber(row)) {
        RO.BasicFun.alertParamCheck("row", "setRowVisible");
    }
    this.grid.setRowVisible(row, visible);
};

/**
* 设置列显示/隐藏
* @param col {Int} 列号
* @visible {Bool} 是否显示
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.prototype.setColVisible = function (col, visible) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    if (!RO.BasicFun.isNumber(col)) {
        RO.BasicFun.alertParamCheck("col", "setColVisible");
    }
    this.grid.setColVisible(col, visible);
};

/**
* 通过数据链接ID获取单元格
* @param {Array} lids 数据链接ID数组
* @param {Object} fkey 浮动数据标识
* @returns {Array} 单元格数组
*/
RO.GridAdapter.prototype.findByDataLinks = function (lids, fkey) {
    if (!RO.BasicFun.isArray(lids)) {
        RO.BasicFun.alertParamCheck("lids", "findByDataLinks");
        return [];
    }
    if (fkey == "null") fkey = null;
    var self = this;
    $(self.grid).unbind("findByDataLinksCallBack").bind("findByDataLinksCallBack", function (e, data) {
        e.stopPropagation();
        var cCells = data.data;
        var cLen = cCells.length;
        var cFindCells = [];
        for (var i = 0; i < cLen; i++) {
            cFindCells.push(new RO.GridAdapter.Cell(cCells[i]));
        }

        $(self.getOptionByName("entry")).trigger("findByDataLinks", { cells: cFindCells });
    });

    var cells = self.grid.findByDataLinks(lids, fkey);
    var len = cells.length;
    var findCells = [];
    for (var i = 0; i < len; i++) {
        findCells.push(new RO.GridAdapter.Cell(cells[i]));
    }

    return findCells;
};

/**
* 获取表格宽度
* @returns {Int} 表格宽度
*/
RO.GridAdapter.prototype.getWidth = function () {
    return this.grid.getWidth() * COM.pixelRatio;
};

/**
* 获取表格高度
* @returns {Int} 表格高度
*/
RO.GridAdapter.prototype.getHeight = function () {
    return this.grid && this.grid.gridModel.gridData.getHeight() * COM.pixelRatio;
};

/**
* 获取表格总列数
* @returns {Int} 表格总列数
*/
RO.GridAdapter.prototype.getColCount = function () {
    return this.grid.getColCount();
};

/**
* 获取表格总行数
* @returns {Int} 表格总行数
*/
RO.GridAdapter.prototype.getRowCount = function () {
    return this.grid.getRowCount();
};

/**
* 获取行高
* @param {Int32} rowIndex 行索引
* @returns {Int} 行高
*/
RO.GridAdapter.prototype.getRowHeight = function (rowIndex) {
    return this.grid.getRowHeight(rowIndex);
};

/**
* 设置行高
* @param {Int32} rowIndex 行索引
* @param {Int32} height 行高
*/
RO.GridAdapter.prototype.setRowHeight = function (rowIndex, height) {
    this.grid.setRowHeight(rowIndex, height);
};

/**
* 获取列宽
* @param {Int32} colIndex 列索引
* @returns {Int} 列宽
*/
RO.GridAdapter.prototype.getColWidth = function (colIndex) {
    return this.grid.getColWidth(colIndex);
};

/**
* 获取列宽
* @param {Int32} colIndex 列索引
* @param {Int32} width 列宽
*/
RO.GridAdapter.prototype.setColWidth = function (colIndex, width) {
    return this.grid.setColWidth(colIndex, width);
};

/**
* 获取某一行里的单元格集合.
* @param {Int32} rowIndex 行索引。
* @returns {Int} 某一行里的单元格集合
*/
RO.GridAdapter.prototype.getRowCells = function (rowIndex) {
    var cells = this.grid.getRowCells(rowIndex);
    var result = [];
    if (cells && cells.length > 0) {
        for (var i = 0, count = cells.length; i < count; i++) {
            result.push(new RO.GridAdapter.Cell(cells[i]));
        }
    }
    return result;
};

/**
* 获取某一列里的单元格集合.
* @param {Int} colIndex 列索引。
* @returns {Int} 某一列里的单元格集合
*/
RO.GridAdapter.prototype.getColCells = function (colIndex) {
    var cells = this.grid.getColCells(colIndex);
    var result = [];
    if (cells && cells.length > 0) {
        for (var i = 0, count = cells.length; i < count; i++) {
            result.push(new RO.GridAdapter.Cell(cells[i]));
        }
    }
    return result;
};

/**
* 获取数据行ExtentInfo属性
* @param {Int} regId 数据区域ID
* @param {Object} fkey 数据行唯一标识
* @returns {Object} ExtentInfo属性值
*/
RO.GridAdapter.prototype.getExtendData = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    return this.grid.getExtendData();
};

/**
* 定位单元格
* 将单元格移至表格窗口居中位置
* @param {RO.Adapter.Cell} cell 待定位单元格
*/
RO.GridAdapter.prototype.center = function (cell) {
    if (!cell) return;
    this.grid.center(cell);
};

/**
* 合并单元格
* @param col {Int} 列号
* @param row {Int} 行号
* @param width {Int} 列数
* @param height {Int} 行数
* @returns {Bool} 是否合并成功
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.prototype.mergeCells = function (col, row, width, height) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return false;
    }
    return !!this.grid.mergeCells(col, row, width, height);
};

/**
* 拆分单元格
* @param col {Int} 列号
* @param row {Int} 行号
* @param force {Bool} 强制拆分模式
* @returns {Bool} 是否拆分成功
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.prototype.unMergeCell = function (col, row, force) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return false;
    }
    return !!this.grid.unMergeCell(col, row, force);
};

/**
* 获取表格中的合并区域
* @returns {Array} sections: [{col,row,width,height}]
*/
RO.GridAdapter.prototype.getMergeSections = function() {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    return this.grid.getMergeSections();
};

/**
* 设置显示隐藏标题栏（0行0列）
* @param showFlag {Boolean} 是否显示
* @returns {Boolean} 是否设置成功
*/
RO.GridAdapter.prototype.setGridTitleBar = function (showFlag) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return false;
    }
    return this.grid.setGridTitleBar(showFlag);
};

/**
* 获取表格纵向滑动距离
* @param step {Int} 行数
* @returns {Int} 滑动距离 
*/
RO.GridAdapter.prototype.getMouseWheelHeight = function(step) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return 0;
    }
    if (isNaN(step)) {
        throw new Error("step参数不合法");
    }
    return this.grid.getMouseWheelHeight(step);
};

/**
* 获取表格横向向滑动距离
* @param step {Int} 列数
* @returns {Int} 滑动距离 
*/
RO.GridAdapter.prototype.getMouseWheelWidth = function (step) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return 0;
    }
    if (isNaN(step)) {
        throw new Error("step参数不合法");
    }
    return this.grid.getMouseWheelWidth(step);
};

/**
* 获取表格纵向滚动位置
*/
RO.GridAdapter.prototype.getScrollTop = function() {
    return this.grid.gridModel.gridData.getScrollTop();
};

/**
*设置表格纵向滚动位置
*/
RO.GridAdapter.prototype.setScrollTop = function (val) {
    if (val >= 0 && !isNaN(val)) {
        this.grid && this.grid.gridModel.gridData.setScrollTop(val);
    }
};

/**
*获取表格横向滚动位置
*/
RO.GridAdapter.prototype.getScrollLeft = function () {
    return this.grid.gridModel.gridData.getScrollLeft();
};

/**
*设置表格纵向滚动位置
*/
RO.GridAdapter.prototype.setScrollLeft = function (val) {
    if (val >= 0 && !isNaN(val)) {
        this.grid && this.grid.gridModel.gridData.setScrollLeft(val);
    }
};

/**
* 获取表格滚动宽度
*/
RO.GridAdapter.prototype.getScrollWidth = function (){
    return this.grid.gridModel.gridData.getScrollWidth();
}
/**
* 获取表格滚动高度
*/
RO.GridAdapter.prototype.getScrollHeight = function (){
    return this.grid && this.grid.gridModel.gridData.getScrollHeight();
}
/**
*获取表格总宽度
*/
RO.GridAdapter.prototype.getDataWidth = function () {
    return this.grid.gridModel.gridData.getDataWidth();
};

/**
*获取表格总高度
*/
RO.GridAdapter.prototype.getDataHeight = function () {
    return this.grid && this.grid.gridModel.gridData.getDataHeight();
};

/**
* 复制
*/
RO.GridAdapter.prototype.copy = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return;
    }
    this.grid.copy();
};

/**
* 粘贴
*/
RO.GridAdapter.prototype.paste = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return;
    }
    this.grid.paste();
};

/**
* 剪切
* @param rect {x,y,width,height} 选区，参数为空，则使用当前表格第一个选区
*/
RO.GridAdapter.prototype.cut = function(rect) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return;
    }
    this.grid.cut(rect);
};

/**
* 获取数据行扩展属性（如reg、state）
* @param name {Int} 扩展属性名
* @param colIndex {Int} 列号【数据区域是按列浮动时生效】
* @param rowIndex {Int} 行号【数据区域是按行浮动时生效】
* @returns {string} 扩展属性
*/
RO.GridAdapter.prototype.getDataRowAttrByName = function(name, colIndex, rowIndex) {
    if (!name) {
        return "";
    }
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return "";
    }
    return this.grid.getDataRowAttrByName(name, colIndex, rowIndex);
};

/**
* 设置数据行扩展属性（如reg、state）
* @param name {Int} 扩展属性名
* @param value {Int} 扩展属性值
* @param colIndex {Int} 列号【数据区域是按列浮动时生效】
* @param rowIndex {Int} 行号【数据区域是按行浮动时生效】
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.prototype.setDataRowAttrByName = function (name, value, colIndex, rowIndex) {
    if (!name) {
        return "";
    }
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return "";
    }
    return this.grid.setDataRowAttrByName(name, value, colIndex, rowIndex);
};

/**
* 获取表格指定相对位置的单元格
* @param clientX {Int} 绝对偏移量
* @param clientY {Int} 绝对偏移量
* @returns {RO.GridAdapter.Cell} 单元格对象
*/
RO.GridAdapter.prototype.getCellByPosition = function (clientX, clientY) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    var cell = this.grid.getCellByPosition(clientX, clientY);
    return cell == null ? null : new RO.GridAdapter.Cell(cell);
};

/**
* 设置表格焦点
* @returns {bool} 是否设置成功
*/
RO.GridAdapter.prototype.focusGrid = function() {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return false;
    }
    return this.grid.focusGrid();
};

/**
* 更新浮动区域数据项fkey
* @param fkeyMaps {origFkey, newFkey} 带更新fkey映射字典
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.prototype.refrashFkey = function(fkeyMaps) {
    this.grid && this.grid.refrashFkey(fkeyMaps);
};

/**
* 更新浮动区域数据项顺序号
* @param fkeyMaps {fkey, floatOrder} fkey为保存后新的rowkey
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.prototype.refrashFloatOrder = function (fkeyMaps) {
    this.grid && this.grid.refrashFloatOrder(fkeyMaps);
};

/**
* 执行数据保存成功后更新表格数据
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.prototype.updateAfterStoreData = function() {
    this.grid && this.grid.updateAfterStoreData();
};

/**
* 获取行所在行浮动区区域内索引
* @param rowIndex {Int} 行号
* @returns {int} 行浮动区区域内索引,若为固定区域则返回当前行号
*/
RO.GridAdapter.prototype.getDataRowIndex = function (rowIndex) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    return this.grid.getDataRowIndex(rowIndex);
};

/**
* 获取行浮动区域数据行第一行号
* @param rid {Int} 浮动区域ID
* @param dataRowIndex {Int} 浮动区域数区域内索引
* @returns {int} 对应行号，若对应的浮动区域不存在则返回0
*/
RO.GridAdapter.prototype.getRowIndexInRegion = function (rid,dataRowIndex) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    return this.grid.getRowIndexInRegion(rid,dataRowIndex);
};

/**
* 获取对应行是否为所在行浮动区区域最后一行
* @param rowIndex {Int} 行号
* @returns {bool} 是否行浮动区域为最后一行
*/
RO.GridAdapter.prototype.isLastRow = function (rowIndex) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    return this.grid.isLastRow(rowIndex);
};

/**
* 获取行浮动区域下一编号区域的起始行号
* @param rowIndex {Int} 浮动区域数区域内索引
* @returns {int} 对应行号，若对应的浮动区域不存在则返回0
*/
RO.GridAdapter.prototype.getNextRowIndex = function (rowIndex) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    return this.grid.getNextRowIndex(rowIndex);
};

/**
* 禁止粘贴操作中新增行列
* @returns {Oject} 当前表格对象
*/
RO.GridAdapter.prototype.denyInsertByPaste = function() {
    this.grid && this.grid.denyInsertByPaste();
    return this;
};

/**
* 根据cell格式化editText为showText
* @param cell {GridCell} 当前要格式化的单元格
* @param editText {String} 要格式化的editText
* @returns {String} editText格式化的结果
*/
RO.GridAdapter.prototype.formatTextByCell = function(cell, editText) {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    if (editText == null || editText == "")
        return "";
    return this.grid.formatTextByCell(cell, editText);
};

/**
* 开始更新表格
*/
RO.GridAdapter.prototype.beginUpdate = function() {
    this.grid && this.grid.beginUpdate();
};

/**
* 完成更新表格
*/
RO.GridAdapter.prototype.endUpdate = function () {
    this.grid && this.grid.endUpdate();
};

/**
* 关闭编辑器
*/
RO.GridAdapter.prototype.closeEditor = function() {
    this.grid && this.grid.closeEditor();
};

/** 
* 打开编辑器
* @param col {Int} 列号
* @param row {Int} 行号
* @param simpleType {Bool} 是否使用简单模式（只提供Open事件）
*/
RO.GridAdapter.prototype.openEditor = function (col, row, simpleType) {
    this.grid && this.grid.openEditor(col, row, simpleType);
};

/**
* 是否正在编辑
* @return {Bool} 返回是否正在编辑
*/
RO.GridAdapter.prototype.isEditing = function () {
    return this.grid && this.grid.isEditing();
};

/**
* 设置表格自定义手势
* @param cursor {String} 手势名称/自定义手势路径，参数为undefined时取消自定义手势
* @returns {RO.GridAdapter} 返回当前表格对象
*/
RO.GridAdapter.prototype.setCustomGridCursor = function (cursor){
    this.grid && this.grid.setCustomGridCursor(cursor);
    return this;
};

/**
* 设置表格自定义手势
* @returns {String} 手势名称/自定义手势路径
*/
RO.GridAdapter.prototype.getCustomGridCursor = function () {
    return this.grid && this.grid.getCustomGridCursor();
};

/**
* 获取表格当前显示区行列范围
* @reuturns {startCol, startRow, endCol, endRow} 
*/
RO.GridAdapter.prototype.getDrawAreaInfo = function () {
    return this.grid && this.grid.getDrawAreaInfo();
};

/**
* 设置表格大小
* @param width {Int} 表格宽度
* @param height {Int} 表格高度
*/
RO.GridAdapter.prototype.setSize = function(width, height) {
    this.grid && this.grid.gridModel.gridData.setSize(width, height);
};

/**
* 设置表格缩放比例
* @param value {Int} 缩放比例
*/
RO.GridAdapter.prototype.setShowProportion = function (value) {
    this.grid && this.grid.setShowProportion(value);
};

/**
* 将表格保存为图片
* @param mineType {String} 图片格式
*/
RO.GridAdapter.prototype.toImg = function (mineType) {
    return this.grid && this.grid.toImg(mineType);
};

/**
* 判断当前单元格数据是否有修改
* 当cellChanged或退出编辑状态时重置
* returns {Bool} 是否有修改
*/
RO.GridAdapter.prototype.isCellValueChanged = function() {
    return this.grid && this.grid.isCellValueChanged();
};
/**
* 销毁GridAdapter对象
*/
RO.GridAdapter.prototype.destroy = function () {
    if (this.grid) {
        if (this.contextMenuSelector) {
            $.contextMenu("destroy", { selector: this.contextMenuSelector });
        }
        if ($("#ztreeInput").length > 0) {
            var zTree = $.fn.zTree.getZTreeObj("treeUl");
            if (zTree) {
                zTree.destroy();
            }
            $("#ztreeInput").combobox("destroy");
            $("#treeEditor").unbind("click");
            $("#treeEditor").remove();
        }

        //todo 临时处理，应该提供一个销毁所有插件的地方.
        var FindAndReplace_dialog = $("#FindAndReplace_dialog");
        if (FindAndReplace_dialog.length) {
            FindAndReplace_dialog.remove();
        }
        
        var Goto_dialog = $("#Goto_dialog");
        if (Goto_dialog.length) {
            Goto_dialog.remove();
        }

        delete this.scrollBar;
        if (RO.GridAdapter.widgetUnReg)
            RO.GridAdapter.widgetUnReg(this);
        this.grid.destroy();
        for (var proto in this) {
            delete this[proto];
        }
        delete this;
    }
};

/*************END****************************/


/**
* @class Cell操作类
* @constructor
* @param cell {Object} 单元格对象
* @returns 返回一个Cell操作类实例
*/
RO.GridAdapter.Cell = function (cell) {
    this.cell = cell;
};

/**
* 单元格属性操作
* 带有value参数为赋值操作，否则为取值
* @param attrName {String} 属性名
* @param value {Object} 属性值
* @returns {Object}
* @exception 表格只读时赋值操作无效
*/
RO.GridAdapter.Cell.prototype.attr = function (attrName, value) {
    if (!RO.BasicFun.isString(attrName)) {
        RO.BasicFun.alertParamCheck("attrName", "attr");
    }
    attrName = attrName.toLowerCase();
    if (attrName == 'class') {
        if (arguments.length == 1) {
            return this.cell.getAttrByName(attrName);
        } else {
            var classSet = this.cell.getAttrByName(attrName);
            var classArray = classSet.split(' ');
            var existedAttr = classArray[value];
            if (existedAttr)
                return value;
            classArray.push(value);
            var result = '';
            for (var i = 0; i < classArray.length; i++) {
                result += classArray[i] + ' ';
            }
            this.cell.setAttrByName(attrName, result);
            return result;
        }
    } else {
        if (arguments.length === 1) {
            return this.cell.getAttrByName(attrName);
        } else {
            this.cell.setAttrByName(attrName, value);
            return value;
        }
    }
};

/**
* 设置单元格是否可编辑
* @param flag {Boolean} 是否可编辑
*/
RO.GridAdapter.Cell.prototype.setCanEdit = function (flag) {
    if (!RO.BasicFun.isBoolean(flag)) {
        RO.BasicFun.alertParamCheck("flag", "setCanEdit");
    }
    this.cell.setCanEdit(flag);
};

/**
* 返回单元格是否可编辑
* @returns {Boolean}
*/
RO.GridAdapter.Cell.prototype.getCanEdit = function () {
    return this.cell.getCanEdit();
};

/**
* 返回单元格ShowText
* @returns {String}
*/
RO.GridAdapter.Cell.prototype.getShowText = function () {
    return this.cell.getShowText();
};

/**
* 单元格赋值
* @param showText {Object} 显示值
* @param editText {Object} 真实值
* @param force {Bool} 忽略只读
* @param customEventData {Object}触发valueChange事件自定义参数customEventData
* @param silence {Bool} 是否静默，true：不触发valuchange事件，false：触发valuechange事件
* @returns {Object}
* @returns {Bool} 普通编辑编号处理，与粘贴等赋值区别开。
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setValue = function (showText, editText, force, customEventData, silence,numFlag) {
    this.cell.setValue(showText, editText, force, customEventData, silence,numFlag);
};

/**
* 返回单元格EditText
* @returns {String}
*/
RO.GridAdapter.Cell.prototype.getEditText = function () {
    return this.cell.getEditText();
};

/**
* 返回单元格行号
* @returns {Int}
*/
RO.GridAdapter.Cell.prototype.getRowIndex = function () {
    return this.cell.getRowIndex();
};

/**
* 返回单元格列号
* @returns {Int}
*/
RO.GridAdapter.Cell.prototype.getColIndex = function () {
    return this.cell.getColIndex();
};

/**
* 返回列合并数
* @returns {Int}
*/
RO.GridAdapter.Cell.prototype.getColSpan = function () {
    return this.cell.getColSpan();
};

/**
* 返回行合并数
* @returns {Int}
*/
RO.GridAdapter.Cell.prototype.getRowSpan = function () {
    return this.cell.getRowSpan();
};

/**
* 返回单元格合并信息
* @returns {Object}
*/
RO.GridAdapter.Cell.prototype.getMergedInfo = function () {
    return this.cell.getMergedInfo();
};


//////////////边框背景样式///////////////////////////
/**
* 设置单元格背景颜色
* @param color {String} 真实值 当参数为空/null时，恢复到初始值
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setBackColor = function (color) {
    this.cell.setBackColor(color);
};

/**
* 获取单元格背景颜色
* @param fromTemplate {bool} 浮动区域时true取表样原始值，否则为数据表初始值
* @returns {String}
*/
RO.GridAdapter.Cell.prototype.getBackColor = function (fromTemplate) {
    return this.cell.getBackColor(fromTemplate);
};

/**
* 设置单元格背景样式
* @param bStyle {RO.Enum.BackStyle} 枚举样式
* @return {RO.GridAdapter.Cell} 返回当前单元格对象
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setBackStyle = function (bStyle) {
    this.cell.setBackStyle(bStyle);
    return this;
};

/**
* 获取单元格背景样式
* @returns {RO.Enum.BackStyle} 枚举样式
*/
RO.GridAdapter.Cell.prototype.getBackStyle = function () {
    return this.cell.getBackStyle();
};

/**
* 设置单元格边框颜色
* @param rColor {String} 右边框颜色 当参数为空/null时，恢复到初始值
* @param bColor {String} 下边框颜色 当参数为空/null时，恢复到初始值
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setBorderColor = function (rColor, bColor) {
    this.cell.setBorderColor(rColor, bColor);
};

/**
* 获取单元格边框颜色
* @returns {Array} [rColor, bColor]
*/
RO.GridAdapter.Cell.prototype.getBorderColor = function () {
    return this.cell.getBorderColor();
};

/**
* 设置边框样式
* @param rStyle {RO.Enum.BorderStyle} 右边框样式 当参数为空/null时，恢复到初始值
* @param bStyle {RO.Enum.BorderStyle} 下边框样式 当参数为空/null时，恢复到初始值
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setBorderStyle = function (rStyle, bStyle) {
    this.cell.setBorderStyle(rStyle, bStyle);
};

/**
* 获取单元格边框样式
* @returns {Array} [rStyle, bStyle]  RO.Enum.BorderStyle枚举样式
*/
RO.GridAdapter.Cell.prototype.getBorderStyle = function () {
    return this.cell.getBorderStyle();
};

/**
* 设置斜线颜色
* @param color {String} 颜色值
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setDiagonalBorderColor = function (color) {
    return this.cell.setDiagonalBorderColor(color);
};


/**
* 设置反斜线
* @param color {String} 颜色值
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setInverseDiagonalBorderColor = function (color) {
    return this.cell.setInverseDiagonalBorderColor(color);
};


/**
* 设置斜线样式
* @param style {Int} 样式
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setDiagonalBorderStyle = function (style) {
    return this.cell.setDiagonalBorderStyle(style);
};


/**
* 设置反斜线样式
* @param style {Int} 样式
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setInverseDiagonalBorderStyle = function (style) {
    return this.cell.setInverseDiagonalBorderStyle(style);
};





/////////////文本布局//////////////////////////////
/**
* 设置单元格水平布局
* @param align {Int} 布局类型值
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setHorzAlign = function (align) {
    this.cell.setHorzAlign(align);
};

/**
* 获取单元格水平布局类型值
* @return {Int} 布局类型值
*/
RO.GridAdapter.Cell.prototype.getHorzAlign = function () {
    return this.cell.getHorzAlign();
};

/**
* 设置单元格垂直布局
* @param align {Int} 布局类型值 COM.widget.Grid.Cell.ALIGN
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setVertAlign = function (align) {
    this.cell.setVertAlign(align);
};

/**
* 获取单元格垂直布局类型值
* @return {Int} 布局类型值
*/
RO.GridAdapter.Cell.prototype.getVertAlign = function () {
    return this.cell.getVertAlign();
};

/**
* 设置单元格垂直文本
* @param value {String} 文本
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setVertText = function (value) {
    this.cell.setVertText(value);
};

/**
* 单元格文字是否是垂直文本
* @return {bool} 是否是垂直文本
*/
RO.GridAdapter.Cell.prototype.isVertText = function () {
    return this.cell.isVertText();
};




///////////////文本显示////////////////////////////////////////

/**
* 获取单元格字体属性
* @return {object}
*/
RO.GridAdapter.Cell.prototype.getFont = function () {
    var fontName = this.cell.getFontName();
    var fontSize = this.cell.getFontSize();
    var fontBold = this.cell.getFontBold();
    var fontItalic = this.cell.getFontItalic();
    var fontColor = this.cell.getFontColor();
    var fontUnderline = this.cell.isUnderline();
    var fontStrikeout = this.cell.isStrikeout();
    return { fontName: fontName, fontSize: fontSize, fontBold: fontBold, fontItalic: fontItalic, fontColor: fontColor, underline: fontUnderline, strikeout: fontStrikeout };
};

/**
* 设置单元格字号
* @param value {String} 字号
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setFontSize = function (value) {
    this.cell.setFontSize(value);
};

/**
* 获取单元格字号
* @return {String} 字号
*/
RO.GridAdapter.Cell.prototype.getFontSize = function () {
    return this.cell.getFontSize();
};

/**
* 设置单元格字体
* @param value {String} 字体
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setFontName = function (value) {
    this.cell.setFontName(value);
};

/**
* 获取单元格字体
* @return {String} 字体
*/
RO.GridAdapter.Cell.prototype.getFontName = function () {
    return this.cell.getFontName();
};

/**
* 设置单元格字体粗细
* @param value {bool} 是否粗体
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setFontBold = function (value) {
    this.cell.setFontBold(value);
};

/**
* 单元格是否设置粗体
* @return {bool} 是否粗体
*/
RO.GridAdapter.Cell.prototype.isFontBold = function () {
    return this.cell.getFontBold();
};

/**
* 设置单元格斜体(火狐浏览器下不支持)
* @param value {bool} 是否斜体
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setFontItalic = function (value) {
    this.cell.setFontItalic(value);
};

/**
* 单元格是否设置斜体(火狐浏览器下不支持)
* @return {bool} 是否斜体
*/
RO.GridAdapter.Cell.prototype.isFontItalic = function () {
    return this.cell.getFontItalic();
};

/**
* 设置单元格文字下划线
* @param value {bool} 是否下划线
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setUnderline = function (value) {
    this.cell.setUnderline(value);
};

/**
* 单元格文字是否设置下划线
* @return {bool} 是否下划线
*/
RO.GridAdapter.Cell.prototype.isUnderline = function () {
    return this.cell.isUnderline();
};

/**
* 设置单元格文字删除线
* @param value {bool} 是否有删除线
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setStrikeout = function (value) {
    this.cell.setStrikeout(value);
};

/**
* 单元格文字是否设置删除线
* @return {bool} 是否有删除线
*/
RO.GridAdapter.Cell.prototype.isStrikeout = function () {
    return this.cell.isStrikeout();
};

/**
* 设置单元格文字颜色
* @param value {String} 颜色
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setFontColor = function (value) {
    this.cell.setFontColor(value);
};

/**
* 获取单元格文字颜色
* @return {String} 颜色
*/
RO.GridAdapter.Cell.prototype.getFontColor = function () {
    return this.cell.getFontColor();
};

/**
* 设置单元格文字自适应
* @param value {bool} 是否自适应
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setFitFontSize = function (value) {
    this.cell.setFitFontSize(value);
};

/**
* 单元格文字是否自适应
* @return {bool} 是否自适应
*/
RO.GridAdapter.Cell.prototype.isFitFontSize = function () {
    return this.cell.isFitFontSize();
};

/**
* 设置单元格文字折行
* @param value {bool} 是否折行
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setWrapLine = function (value) {
    this.cell.setWrapLine(value);
};

/**
* 单元格文字是否折行
* @return {bool} 是否折行
*/
RO.GridAdapter.Cell.prototype.isWrapLine = function () {
    return this.cell.isWrapLine();
};

/**
* 设置单元格文字多行
* @param value {bool} 是否多行显示
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setMultiLine = function (value) {
    this.cell.setMultiLine(value);
};

/**
* 单元格文字是否多行显示
* @return {bool} 是否多行显示
*/
RO.GridAdapter.Cell.prototype.isMultiLine = function () {
    return this.cell.isMultiLine();
};

/**
* 设置单元格文字缩进
* @param indent {String} 缩进值（单位宽度：2字符）
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setIndent = function (indent) {
    this.cell.setIndent(indent);
};

/**
* 获取单元格文字缩进
* @return {String} 缩进值（单位宽度：2字符）
*/
RO.GridAdapter.Cell.prototype.getIndent = function () {
    return this.cell.getIndent();
};

/**
* 设置单元格文字缩进
* @param indentPx {String} 缩进值（单位宽度：1像素）
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setIndentPx = function (indentPx) {
    this.cell.setIndentPx(indentPx);
};

/**
* 获取单元格文字缩进
* @return {String} 缩进值（单位宽度：1像素）
*/
RO.GridAdapter.Cell.prototype.getIndentPx = function () {
    return this.cell.getIndentPx();
};

/**
* 设置单元格提示信息
* @param value {String} 提示信息
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setCellTip = function (value) {
    this.cell.setCellTip(value);
};

/**
* 获取单元格绝对偏移
* @returns {x,y} 单元格偏移
*/
RO.GridAdapter.Cell.prototype.getPosition = function () {
    return this.cell.getPosition();
};

/**
* 设置图片
* @param expand {name} 图片类型名称（如审核图片，校验图片）
* @param expand {options} 参数（img ，bounds ，margin）
* @param expand {fixRowHeight} 自适应，默认false
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setImgIntoCell = function (name, options, fixRowHeight) {
    this.cell.setImgIntoCell(name, options, fixRowHeight);
};

/**
* 设置前景色
* @param foreColor {String} 前景色
* @return {RO.GridAdapter.Cell} 当前单元格对象
* @exception 表格只读时该操作无效
*/
RO.GridAdapter.Cell.prototype.setForeColor = function (foreColor) {
    this.cell.setForeColor(foreColor);
    return this;
};

/**
* 获取前景色
* @return {String} 前景色
*/
RO.GridAdapter.Cell.prototype.getForeColor = function () {
    return this.cell.getForeColor();
};




/******表格扩展方法******************/


///////枚举数据源扩展 //////
RO.GridAdapter.extend(RO.GridAdapter, {

    // 请求数据源的时机由业务模块来控制，grid加载枚举的时候只从缓存中加载
    // 
    requestEnumNodes: function () {
        //if (this.grid == null || this.grid.gridModel == null)
        //    return;
        //var me = this;
        //setTimeout(function () {
        //    me._requestEnumNodes();
        //}, 0);
    },

    setEnumNodes: function (data) {
        if (data)
            RO.Enum.EnumNodesCache = data;
    },

    _requestEnumNodes: function (enumId) {
        var sourceUrl = this.grid.gridModel.enumDataSourceUrl;
        var enumDataBlobUrl = null;//this.grid.gridModel.enumDataBlobUrl;
        var grid = this.grid;
        if (enumDataBlobUrl) {
            enumDataBlobUrl +=enumId;
            $.ajax({
                url: enumDataBlobUrl,
                dataType: 'jsonp',
                jsonpCallback: 'dataCallback',
                success: function (treeData) {
                    if (treeData) {
                        RO.Enum.EnumNodesCache = treeData;
                        $(grid.gridModel.entry).trigger("LoadEnumData");
                    }
                },
                error: function (XMLHttpRequest, textStatus, errorThrown) {

                }
            });
        } else if (sourceUrl) {
        	var url = $.BaseUrl.substring(0, $.BaseUrl.length-1) + sourceUrl + "&enumId=" + enumId;
        	url = this._doUrl(url);
            $.ajax({
                type: "post",
                url: url,
                contentType: "application/json",
                success: function (treeData) {
                    if (treeData) {
                        RO.Enum.EnumNodesCache = treeData;
                        $(grid.gridModel.entry).trigger("LoadEnumData");
                    }
                },
                error: function (XMLHttpRequest, textStatus, errorThrown) {

                }
            });
        }
    },
    
    _doUrl: function(url){
    	var location = window.top.location;
		var taskIdIndex = location.search.indexOf("_taskId_");
		if (taskIdIndex !== -1) {
			var reg = new RegExp('(^|&)' + "_taskId_" + '=([^&]*)(&|$)', 'i'),
				reg3 = new RegExp('(^|&)' + "shareToken" + '=([^&]*)(&|$)', 'i'),
				reg5 = new RegExp('(^|&)' + "InviteKey" + '=([^&]*)(&|$)', 'i'),
				r = location.search.substr(1).match(reg),
				r3 = location.search.substr(1).match(reg3),
				r5 = location.search.substr(1).match(reg5);
			if (r != null && url.indexOf("_taskId_") === -1) // _taskId_
				url += url.indexOf("?") !== -1 ? "&_taskId_=" + unescape(r[2]) : "?_taskId_=" + unescape(r[2]);
			if (r3 != null && url.indexOf("shareToken") === -1) // shareToken
				url += url.indexOf("?") !== -1 ? "&shareToken=" + unescape(r3[2]) : "?shareToken=" + unescape(r3[2]);
			if (r5 != null && url.indexOf("InviteKey") === -1) // InviteKey
				url += url.indexOf("?") !== -1 ? "&InviteKey=" + unescape(r5[2]) : "?InviteKey=" + unescape(r5[2]);
		}
		return url;
    }
});
